@@ -7,20 +7,19 @@ use cosmwasm_std::{
     Response, StdResult,
 };
 
-use cw0::{maybe_addr, Expiration};
 use cw2::set_contract_version;
 use cw3::{
-    ProposalListResponse, ProposalResponse, Status, ThresholdResponse, Vote, VoteInfo,
-    VoteListResponse, VoteResponse, VoterDetail, VoterListResponse, VoterResponse,
+    ProposalListResponse, ProposalResponse, Status, Vote, VoteInfo, VoteListResponse, VoteResponse,
+    VoterDetail, VoterListResponse, VoterResponse,
 };
+use cw3_fixed_multisig::state::{next_id, Ballot, Proposal, Votes, BALLOTS, PROPOSALS};
 use cw4::{Cw4Contract, MemberChangedHookMsg, MemberDiff};
 use cw_storage_plus::Bound;
+use cw_utils::{maybe_addr, Expiration, ThresholdResponse};
 
 use crate::error::ContractError;
 use crate::msg::{ExecuteMsg, InstantiateMsg, QueryMsg};
-use crate::state::{
-    next_id, parse_id, Ballot, Config, Proposal, Votes, BALLOTS, CONFIG, PROPOSALS,
-};
+use crate::state::{Config, CONFIG};
 
 // version info for migration info
 const CONTRACT_NAME: &str = "crates.io:cw3-flex-multisig";
@@ -89,9 +88,14 @@ pub fn execute_propose(
     // only members of the multisig can create a proposal
     let cfg = CONFIG.load(deps.storage)?;
 
+    // Only members of the multisig can create a proposal
+    // Non-voting members are special - they are allowed to create a proposal and
+    // therefore "vote", but they aren't allowed to vote otherwise.
+    // Such vote is also special, because despite having 0 weight it still counts when
+    // counting threshold passing
     let vote_power = cfg
         .group_addr
-        .is_member(&deps.querier, &info.sender)?
+        .is_member(&deps.querier, &info.sender, None)?
         .ok_or(ContractError::Unauthorized {})?;
 
     // max expires also used as default
@@ -112,20 +116,20 @@ pub fn execute_propose(
         expires,
         msgs,
         status: Status::Open,
-        votes: Votes::new(vote_power),
+        votes: Votes::yes(vote_power),
         threshold: cfg.threshold,
         total_weight: cfg.group_addr.total_weight(&deps.querier)?,
     };
     prop.update_status(&env.block);
     let id = next_id(deps.storage)?;
-    PROPOSALS.save(deps.storage, id.into(), &prop)?;
+    PROPOSALS.save(deps.storage, id, &prop)?;
 
     // add the first yes vote from voter
     let ballot = Ballot {
         weight: vote_power,
         vote: Vote::Yes,
     };
-    BALLOTS.save(deps.storage, (id.into(), &info.sender), &ballot)?;
+    BALLOTS.save(deps.storage, (id, &info.sender), &ballot)?;
 
     Ok(Response::new()
         .add_attribute("action", "propose")
@@ -145,7 +149,7 @@ pub fn execute_vote(
     let cfg = CONFIG.load(deps.storage)?;
 
     // ensure proposal exists and can be voted on
-    let mut prop = PROPOSALS.load(deps.storage, proposal_id.into())?;
+    let mut prop = PROPOSALS.load(deps.storage, proposal_id)?;
     if prop.status != Status::Open {
         return Err(ContractError::NotOpen {});
     }
@@ -153,29 +157,27 @@ pub fn execute_vote(
         return Err(ContractError::Expired {});
     }
 
+    // Only voting members of the multisig can vote
+    // Additional check if weight >= 1
     // use a snapshot of "start of proposal"
     let vote_power = cfg
         .group_addr
-        .member_at_height(&deps.querier, info.sender.clone(), prop.start_height)?
+        .is_voting_member(&deps.querier, &info.sender, prop.start_height)?
         .ok_or(ContractError::Unauthorized {})?;
 
     // cast vote if no vote previously cast
-    BALLOTS.update(
-        deps.storage,
-        (proposal_id.into(), &info.sender),
-        |bal| match bal {
-            Some(_) => Err(ContractError::AlreadyVoted {}),
-            None => Ok(Ballot {
-                weight: vote_power,
-                vote,
-            }),
-        },
-    )?;
+    BALLOTS.update(deps.storage, (proposal_id, &info.sender), |bal| match bal {
+        Some(_) => Err(ContractError::AlreadyVoted {}),
+        None => Ok(Ballot {
+            weight: vote_power,
+            vote,
+        }),
+    })?;
 
     // update vote tally
     prop.votes.add_vote(vote, vote_power);
     prop.update_status(&env.block);
-    PROPOSALS.save(deps.storage, proposal_id.into(), &prop)?;
+    PROPOSALS.save(deps.storage, proposal_id, &prop)?;
 
     Ok(Response::new()
         .add_attribute("action", "vote")
@@ -186,22 +188,22 @@ pub fn execute_vote(
 
 pub fn execute_execute(
     deps: DepsMut,
-    _env: Env,
+    env: Env,
     info: MessageInfo,
     proposal_id: u64,
 ) -> Result<Response, ContractError> {
     // anyone can trigger this if the vote passed
 
-    let mut prop = PROPOSALS.load(deps.storage, proposal_id.into())?;
+    let mut prop = PROPOSALS.load(deps.storage, proposal_id)?;
     // we allow execution even after the proposal "expiration" as long as all vote come in before
     // that point. If it was approved on time, it can be executed any time.
-    if prop.status != Status::Passed {
+    if prop.current_status(&env.block) != Status::Passed {
         return Err(ContractError::WrongExecuteStatus {});
     }
 
     // set it to executed
     prop.status = Status::Executed;
-    PROPOSALS.save(deps.storage, proposal_id.into(), &prop)?;
+    PROPOSALS.save(deps.storage, proposal_id, &prop)?;
 
     // dispatch all proposed messages
     Ok(Response::new()
@@ -219,7 +221,7 @@ pub fn execute_close(
 ) -> Result<Response<Empty>, ContractError> {
     // anyone can trigger this if the vote passed
 
-    let mut prop = PROPOSALS.load(deps.storage, proposal_id.into())?;
+    let mut prop = PROPOSALS.load(deps.storage, proposal_id)?;
     if [Status::Executed, Status::Rejected, Status::Passed]
         .iter()
         .any(|x| *x == prop.status)
@@ -232,7 +234,7 @@ pub fn execute_close(
 
     // set it to failed
     prop.status = Status::Rejected;
-    PROPOSALS.save(deps.storage, proposal_id.into(), &prop)?;
+    PROPOSALS.save(deps.storage, proposal_id, &prop)?;
 
     Ok(Response::new()
         .add_attribute("action", "close")
@@ -288,7 +290,7 @@ fn query_threshold(deps: Deps) -> StdResult<ThresholdResponse> {
 }
 
 fn query_proposal(deps: Deps, env: Env, id: u64) -> StdResult<ProposalResponse> {
-    let prop = PROPOSALS.load(deps.storage, id.into())?;
+    let prop = PROPOSALS.load(deps.storage, id)?;
     let status = prop.current_status(&env.block);
     let threshold = prop.threshold.to_response(prop.total_weight);
     Ok(ProposalResponse {
@@ -313,14 +315,14 @@ fn list_proposals(
     limit: Option<u32>,
 ) -> StdResult<ProposalListResponse> {
     let limit = limit.unwrap_or(DEFAULT_LIMIT).min(MAX_LIMIT) as usize;
-    let start = start_after.map(Bound::exclusive_int);
-    let props: StdResult<Vec<_>> = PROPOSALS
+    let start = start_after.map(Bound::exclusive);
+    let proposals = PROPOSALS
         .range(deps.storage, start, None, Order::Ascending)
         .take(limit)
         .map(|p| map_proposal(&env.block, p))
-        .collect();
+        .collect::<StdResult<_>>()?;
 
-    Ok(ProposalListResponse { proposals: props? })
+    Ok(ProposalListResponse { proposals })
 }
 
 fn reverse_proposals(
@@ -330,7 +332,7 @@ fn reverse_proposals(
     limit: Option<u32>,
 ) -> StdResult<ProposalListResponse> {
     let limit = limit.unwrap_or(DEFAULT_LIMIT).min(MAX_LIMIT) as usize;
-    let end = start_before.map(Bound::exclusive_int);
+    let end = start_before.map(Bound::exclusive);
     let props: StdResult<Vec<_>> = PROPOSALS
         .range(deps.storage, None, end, Order::Descending)
         .take(limit)
@@ -342,26 +344,28 @@ fn reverse_proposals(
 
 fn map_proposal(
     block: &BlockInfo,
-    item: StdResult<(Vec<u8>, Proposal)>,
+    item: StdResult<(u64, Proposal)>,
 ) -> StdResult<ProposalResponse> {
-    let (key, prop) = item?;
-    let status = prop.current_status(block);
-    let threshold = prop.threshold.to_response(prop.total_weight);
-    Ok(ProposalResponse {
-        id: parse_id(&key)?,
-        title: prop.title,
-        description: prop.description,
-        msgs: prop.msgs,
-        status,
-        expires: prop.expires,
-        threshold,
+    item.map(|(id, prop)| {
+        let status = prop.current_status(block);
+        let threshold = prop.threshold.to_response(prop.total_weight);
+        ProposalResponse {
+            id,
+            title: prop.title,
+            description: prop.description,
+            msgs: prop.msgs,
+            status,
+            expires: prop.expires,
+            threshold,
+        }
     })
 }
 
 fn query_vote(deps: Deps, proposal_id: u64, voter: String) -> StdResult<VoteResponse> {
     let voter_addr = deps.api.addr_validate(&voter)?;
-    let prop = BALLOTS.may_load(deps.storage, (proposal_id.into(), &voter_addr))?;
+    let prop = BALLOTS.may_load(deps.storage, (proposal_id, &voter_addr))?;
     let vote = prop.map(|b| VoteInfo {
+        proposal_id,
         voter,
         vote: b.vote,
         weight: b.weight,
@@ -377,29 +381,29 @@ fn list_votes(
 ) -> StdResult<VoteListResponse> {
     let limit = limit.unwrap_or(DEFAULT_LIMIT).min(MAX_LIMIT) as usize;
     let addr = maybe_addr(deps.api, start_after)?;
-    let start = addr.map(|addr| Bound::exclusive(addr.as_ref()));
+    let start = addr.as_ref().map(Bound::exclusive);
 
-    let votes: StdResult<Vec<_>> = BALLOTS
-        .prefix(proposal_id.into())
+    let votes = BALLOTS
+        .prefix(proposal_id)
         .range(deps.storage, start, None, Order::Ascending)
         .take(limit)
         .map(|item| {
-            let (voter, ballot) = item?;
-            Ok(VoteInfo {
-                voter: String::from_utf8(voter)?,
+            item.map(|(addr, ballot)| VoteInfo {
+                proposal_id,
+                voter: addr.into(),
                 vote: ballot.vote,
                 weight: ballot.weight,
             })
         })
-        .collect();
+        .collect::<StdResult<_>>()?;
 
-    Ok(VoteListResponse { votes: votes? })
+    Ok(VoteListResponse { votes })
 }
 
 fn query_voter(deps: Deps, voter: String) -> StdResult<VoterResponse> {
     let cfg = CONFIG.load(deps.storage)?;
     let voter_addr = deps.api.addr_validate(&voter)?;
-    let weight = cfg.group_addr.is_member(&deps.querier, &voter_addr)?;
+    let weight = cfg.group_addr.is_member(&deps.querier, &voter_addr, None)?;
 
     Ok(VoterResponse { weight })
 }
@@ -426,14 +430,13 @@ fn list_voters(
 mod tests {
     use cosmwasm_std::{coin, coins, Addr, BankMsg, Coin, Decimal, Timestamp};
 
-    use cw0::Duration;
     use cw2::{query_contract_info, ContractVersion};
     use cw4::{Cw4ExecuteMsg, Member};
     use cw4_group::helpers::Cw4GroupContract;
     use cw_multi_test::{next_block, App, AppBuilder, Contract, ContractWrapper, Executor};
+    use cw_utils::{Duration, Threshold};
 
     use super::*;
-    use crate::msg::Threshold;
 
     const OWNER: &str = "admin0001";
     const VOTER1: &str = "voter0001";
@@ -488,6 +491,7 @@ mod tests {
             .unwrap()
     }
 
+    #[track_caller]
     fn instantiate_flex(
         app: &mut App,
         group: Addr,
@@ -507,6 +511,7 @@ mod tests {
     // this will set up both contracts, instantiating the group with
     // all voters defined above, and the multisig pointing to it and given threshold criteria.
     // Returns (multisig address, group address).
+    #[track_caller]
     fn setup_test_case_fixed(
         app: &mut App,
         weight_needed: u64,
@@ -525,6 +530,7 @@ mod tests {
         )
     }
 
+    #[track_caller]
     fn setup_test_case(
         app: &mut App,
         threshold: Threshold,
@@ -538,7 +544,7 @@ mod tests {
             member(VOTER1, 1),
             member(VOTER2, 2),
             member(VOTER3, 3),
-            member(VOTER4, 4),
+            member(VOTER4, 12), // so that he alone can pass a 50 / 52% threshold proposal
             member(VOTER5, 5),
         ];
         let group_addr = instantiate_group(app, members);
@@ -605,7 +611,10 @@ mod tests {
         // Zero required weight fails
         let instantiate_msg = InstantiateMsg {
             group_addr: group_addr.to_string(),
-            threshold: Threshold::AbsoluteCount { weight: 0 },
+            threshold: Threshold::ThresholdQuorum {
+                threshold: Decimal::zero(),
+                quorum: Decimal::percent(1),
+            },
             max_voting_period,
         };
         let err = app
@@ -618,7 +627,10 @@ mod tests {
                 None,
             )
             .unwrap_err();
-        assert_eq!(ContractError::ZeroThreshold {}, err.downcast().unwrap());
+        assert_eq!(
+            ContractError::Threshold(cw_utils::ThresholdError::InvalidThreshold {}),
+            err.downcast().unwrap()
+        );
 
         // Total weight less than required weight not allowed
         let instantiate_msg = InstantiateMsg {
@@ -637,7 +649,7 @@ mod tests {
             )
             .unwrap_err();
         assert_eq!(
-            ContractError::UnreachableThreshold {},
+            ContractError::Threshold(cw_utils::ThresholdError::UnreachableWeight {}),
             err.downcast().unwrap()
         );
 
@@ -798,10 +810,12 @@ mod tests {
         let init_funds = coins(10, "BTC");
         let mut app = mock_app(&init_funds);
 
-        let required_weight = 3;
         let voting_period = Duration::Time(2000000);
-        let (flex_addr, _) =
-            setup_test_case_fixed(&mut app, required_weight, voting_period, init_funds, false);
+        let threshold = Threshold::ThresholdQuorum {
+            threshold: Decimal::percent(80),
+            quorum: Decimal::percent(20),
+        };
+        let (flex_addr, _) = setup_test_case(&mut app, threshold, voting_period, init_funds, false);
 
         // create proposal with 1 vote power
         let proposal = pay_somebody_proposal();
@@ -814,7 +828,7 @@ mod tests {
         app.update_block(next_block);
         let proposal = pay_somebody_proposal();
         let res = app
-            .execute_contract(Addr::unchecked(VOTER3), flex_addr.clone(), &proposal, &[])
+            .execute_contract(Addr::unchecked(VOTER4), flex_addr.clone(), &proposal, &[])
             .unwrap();
         let proposal_id2: u64 = res.custom_attrs(1)[2].value.parse().unwrap();
 
@@ -877,9 +891,10 @@ mod tests {
             msgs,
             expires: voting_period.after(&proposed_at),
             status: Status::Open,
-            threshold: ThresholdResponse::AbsoluteCount {
-                weight: 3,
-                total_weight: 15,
+            threshold: ThresholdResponse::ThresholdQuorum {
+                total_weight: 23,
+                threshold: Decimal::percent(80),
+                quorum: Decimal::percent(20),
             },
         };
         assert_eq!(&expected, &res.proposals[0]);
@@ -890,10 +905,12 @@ mod tests {
         let init_funds = coins(10, "BTC");
         let mut app = mock_app(&init_funds);
 
-        let required_weight = 3;
+        let threshold = Threshold::ThresholdQuorum {
+            threshold: Decimal::percent(51),
+            quorum: Decimal::percent(1),
+        };
         let voting_period = Duration::Time(2000000);
-        let (flex_addr, _) =
-            setup_test_case_fixed(&mut app, required_weight, voting_period, init_funds, false);
+        let (flex_addr, _) = setup_test_case(&mut app, threshold, voting_period, init_funds, false);
 
         // create proposal with 0 vote power
         let proposal = pay_somebody_proposal();
@@ -904,7 +921,7 @@ mod tests {
         // Get the proposal id from the logs
         let proposal_id: u64 = res.custom_attrs(1)[2].value.parse().unwrap();
 
-        // Owner cannot vote (again)
+        // Owner with 0 voting power cannot vote
         let yes_vote = ExecuteMsg::Vote {
             proposal_id,
             vote: Vote::Yes,
@@ -912,7 +929,7 @@ mod tests {
         let err = app
             .execute_contract(Addr::unchecked(OWNER), flex_addr.clone(), &yes_vote, &[])
             .unwrap_err();
-        assert_eq!(ContractError::AlreadyVoted {}, err.downcast().unwrap());
+        assert_eq!(ContractError::Unauthorized {}, err.downcast().unwrap());
 
         // Only voters can vote
         let err = app
@@ -934,6 +951,12 @@ mod tests {
             ],
         );
 
+        // VOTER1 cannot vote again
+        let err = app
+            .execute_contract(Addr::unchecked(VOTER1), flex_addr.clone(), &yes_vote, &[])
+            .unwrap_err();
+        assert_eq!(ContractError::AlreadyVoted {}, err.downcast().unwrap());
+
         // No/Veto votes have no effect on the tally
         // Compute the current tally
         let tally = get_tally(&app, flex_addr.as_ref(), proposal_id);
@@ -1003,6 +1026,7 @@ mod tests {
         assert_eq!(
             vote.vote.unwrap(),
             VoteInfo {
+                proposal_id,
                 voter: OWNER.into(),
                 vote: Vote::Yes,
                 weight: 0
@@ -1018,6 +1042,7 @@ mod tests {
         assert_eq!(
             vote.vote.unwrap(),
             VoteInfo {
+                proposal_id,
                 voter: VOTER2.into(),
                 vote: Vote::No,
                 weight: 2
@@ -1038,10 +1063,12 @@ mod tests {
         let init_funds = coins(10, "BTC");
         let mut app = mock_app(&init_funds);
 
-        let required_weight = 3;
+        let threshold = Threshold::ThresholdQuorum {
+            threshold: Decimal::percent(51),
+            quorum: Decimal::percent(1),
+        };
         let voting_period = Duration::Time(2000000);
-        let (flex_addr, _) =
-            setup_test_case_fixed(&mut app, required_weight, voting_period, init_funds, true);
+        let (flex_addr, _) = setup_test_case(&mut app, threshold, voting_period, init_funds, true);
 
         // ensure we have cash to cover the proposal
         let contract_bal = app.wrap().query_balance(&flex_addr, "BTC").unwrap();
@@ -1072,13 +1099,13 @@ mod tests {
             vote: Vote::Yes,
         };
         let res = app
-            .execute_contract(Addr::unchecked(VOTER3), flex_addr.clone(), &vote, &[])
+            .execute_contract(Addr::unchecked(VOTER4), flex_addr.clone(), &vote, &[])
             .unwrap();
         assert_eq!(
             res.custom_attrs(1),
             [
                 ("action", "vote"),
-                ("sender", VOTER3),
+                ("sender", VOTER4),
                 ("proposal_id", proposal_id.to_string().as_str()),
                 ("status", "Passed"),
             ],
@@ -1117,9 +1144,99 @@ mod tests {
 
         // In passing: Try to close Executed fails
         let err = app
-            .execute_contract(Addr::unchecked(OWNER), flex_addr, &closing, &[])
+            .execute_contract(Addr::unchecked(OWNER), flex_addr.clone(), &closing, &[])
             .unwrap_err();
         assert_eq!(ContractError::WrongCloseStatus {}, err.downcast().unwrap());
+
+        // Trying to execute something that was already executed fails
+        let err = app
+            .execute_contract(Addr::unchecked(SOMEBODY), flex_addr, &execution, &[])
+            .unwrap_err();
+        assert_eq!(
+            ContractError::WrongExecuteStatus {},
+            err.downcast().unwrap()
+        );
+    }
+
+    #[test]
+    fn proposal_pass_on_expiration() {
+        let init_funds = coins(10, "BTC");
+        let mut app = mock_app(&init_funds);
+
+        let threshold = Threshold::ThresholdQuorum {
+            threshold: Decimal::percent(51),
+            quorum: Decimal::percent(1),
+        };
+        let voting_period = 2000000;
+        let (flex_addr, _) = setup_test_case(
+            &mut app,
+            threshold,
+            Duration::Time(voting_period),
+            init_funds,
+            true,
+        );
+
+        // ensure we have cash to cover the proposal
+        let contract_bal = app.wrap().query_balance(&flex_addr, "BTC").unwrap();
+        assert_eq!(contract_bal, coin(10, "BTC"));
+
+        // create proposal with 0 vote power
+        let proposal = pay_somebody_proposal();
+        let res = app
+            .execute_contract(Addr::unchecked(OWNER), flex_addr.clone(), &proposal, &[])
+            .unwrap();
+
+        // Get the proposal id from the logs
+        let proposal_id: u64 = res.custom_attrs(1)[2].value.parse().unwrap();
+
+        // Vote it, so it passes after voting period is over
+        let vote = ExecuteMsg::Vote {
+            proposal_id,
+            vote: Vote::Yes,
+        };
+        let res = app
+            .execute_contract(Addr::unchecked(VOTER3), flex_addr.clone(), &vote, &[])
+            .unwrap();
+        assert_eq!(
+            res.custom_attrs(1),
+            [
+                ("action", "vote"),
+                ("sender", VOTER3),
+                ("proposal_id", proposal_id.to_string().as_str()),
+                ("status", "Open"),
+            ],
+        );
+
+        // Wait until the voting period is over.
+        app.update_block(|block| {
+            block.time = block.time.plus_seconds(voting_period);
+            block.height += std::cmp::max(1, voting_period / 5);
+        });
+
+        // Proposal should now be passed.
+        let prop: ProposalResponse = app
+            .wrap()
+            .query_wasm_smart(&flex_addr, &QueryMsg::Proposal { proposal_id })
+            .unwrap();
+        assert_eq!(prop.status, Status::Passed);
+
+        // Execution should now be possible.
+        let res = app
+            .execute_contract(
+                Addr::unchecked(SOMEBODY),
+                flex_addr,
+                &ExecuteMsg::Execute { proposal_id },
+                &[],
+            )
+            .unwrap();
+        assert_eq!(
+            res.custom_attrs(1),
+            [
+                ("action", "execute"),
+                ("sender", SOMEBODY),
+                ("proposal_id", proposal_id.to_string().as_str()),
+            ],
+        );
     }
 
     #[test]
@@ -1127,10 +1244,12 @@ mod tests {
         let init_funds = coins(10, "BTC");
         let mut app = mock_app(&init_funds);
 
-        let required_weight = 3;
+        let threshold = Threshold::ThresholdQuorum {
+            threshold: Decimal::percent(51),
+            quorum: Decimal::percent(1),
+        };
         let voting_period = Duration::Height(2000000);
-        let (flex_addr, _) =
-            setup_test_case_fixed(&mut app, required_weight, voting_period, init_funds, true);
+        let (flex_addr, _) = setup_test_case(&mut app, threshold, voting_period, init_funds, true);
 
         // create proposal with 0 vote power
         let proposal = pay_somebody_proposal();
@@ -1176,10 +1295,13 @@ mod tests {
         let init_funds = coins(10, "BTC");
         let mut app = mock_app(&init_funds);
 
-        let required_weight = 4;
+        let threshold = Threshold::ThresholdQuorum {
+            threshold: Decimal::percent(51),
+            quorum: Decimal::percent(1),
+        };
         let voting_period = Duration::Time(20000);
         let (flex_addr, group_addr) =
-            setup_test_case_fixed(&mut app, required_weight, voting_period, init_funds, false);
+            setup_test_case(&mut app, threshold, voting_period, init_funds, false);
 
         // VOTER1 starts a proposal to send some tokens (1/4 votes)
         let proposal = pay_somebody_proposal();
@@ -1205,9 +1327,10 @@ mod tests {
             .wrap()
             .query_wasm_smart(&flex_addr, &QueryMsg::Threshold {})
             .unwrap();
-        let expected_thresh = ThresholdResponse::AbsoluteCount {
-            weight: 4,
-            total_weight: 15,
+        let expected_thresh = ThresholdResponse::ThresholdQuorum {
+            total_weight: 23,
+            threshold: Decimal::percent(51),
+            quorum: Decimal::percent(1),
         };
         assert_eq!(expected_thresh, threshold);
 
@@ -1215,13 +1338,13 @@ mod tests {
         app.update_block(|block| block.height += 2);
 
         // admin changes the group
-        // updates VOTER2 power to 7 -> with snapshot, vote doesn't pass proposal
+        // updates VOTER2 power to 21 -> with snapshot, vote doesn't pass proposal
         // adds NEWBIE with 2 power -> with snapshot, invalid vote
-        // removes VOTER3 -> with snapshot, can vote and pass proposal
+        // removes VOTER3 -> with snapshot, can vote on proposal
         let newbie: &str = "newbie";
         let update_msg = cw4_group::msg::ExecuteMsg::UpdateMembers {
             remove: vec![VOTER3.into()],
-            add: vec![member(VOTER2, 7), member(newbie, 2)],
+            add: vec![member(VOTER2, 21), member(newbie, 2)],
         };
         app.execute_contract(Addr::unchecked(OWNER), group_addr, &update_msg, &[])
             .unwrap();
@@ -1277,16 +1400,16 @@ mod tests {
         // previously removed VOTER3 can still vote, passing the proposal
         app.execute_contract(Addr::unchecked(VOTER3), flex_addr.clone(), &yes_vote, &[])
             .unwrap();
-        assert_eq!(prop_status(&app, proposal_id), Status::Passed);
 
         // check current threshold (global) is updated
         let threshold: ThresholdResponse = app
             .wrap()
             .query_wasm_smart(&flex_addr, &QueryMsg::Threshold {})
             .unwrap();
-        let expected_thresh = ThresholdResponse::AbsoluteCount {
-            weight: 4,
-            total_weight: 19,
+        let expected_thresh = ThresholdResponse::ThresholdQuorum {
+            total_weight: 41,
+            threshold: Decimal::percent(51),
+            quorum: Decimal::percent(1),
         };
         assert_eq!(expected_thresh, threshold);
 
@@ -1417,19 +1540,16 @@ mod tests {
         let init_funds = coins(10, "BTC");
         let mut app = mock_app(&init_funds);
 
-        // 33% required, which is 5 of the initial 15
+        // 51% required, which is 12 of the initial 24
+        let threshold = Threshold::ThresholdQuorum {
+            threshold: Decimal::percent(51),
+            quorum: Decimal::percent(1),
+        };
         let voting_period = Duration::Time(20000);
-        let (flex_addr, group_addr) = setup_test_case(
-            &mut app,
-            Threshold::AbsolutePercentage {
-                percentage: Decimal::percent(33),
-            },
-            voting_period,
-            init_funds,
-            false,
-        );
+        let (flex_addr, group_addr) =
+            setup_test_case(&mut app, threshold, voting_period, init_funds, false);
 
-        // VOTER3 starts a proposal to send some tokens (3/5 votes)
+        // VOTER3 starts a proposal to send some tokens (3/12 votes)
         let proposal = pay_somebody_proposal();
         let res = app
             .execute_contract(Addr::unchecked(VOTER3), flex_addr.clone(), &proposal, &[])
@@ -1445,17 +1565,17 @@ mod tests {
             prop.status
         };
 
-        // 3/5 votes
+        // 3/12 votes
         assert_eq!(prop_status(&app), Status::Open);
 
         // a few blocks later...
         app.update_block(|block| block.height += 2);
 
-        // admin changes the group (3 -> 0, 2 -> 7, 0 -> 15) - total = 32, require 11 to pass
+        // admin changes the group (3 -> 0, 2 -> 9, 0 -> 29) - total = 56, require 29 to pass
         let newbie: &str = "newbie";
         let update_msg = cw4_group::msg::ExecuteMsg::UpdateMembers {
             remove: vec![VOTER3.into()],
-            add: vec![member(VOTER2, 7), member(newbie, 15)],
+            add: vec![member(VOTER2, 9), member(newbie, 29)],
         };
         app.execute_contract(Addr::unchecked(OWNER), group_addr, &update_msg, &[])
             .unwrap();
@@ -1463,15 +1583,15 @@ mod tests {
         // a few blocks later...
         app.update_block(|block| block.height += 3);
 
-        // VOTER2 votes according to original weights: 3 + 2 = 5 / 5 => Passed
-        // with updated weights, it would be 3 + 7 = 10 / 11 => Open
+        // VOTER2 votes according to original weights: 3 + 2 = 5 / 12 => Open
+        // with updated weights, it would be 3 + 9 = 12 / 12 => Passed
         let yes_vote = ExecuteMsg::Vote {
             proposal_id,
             vote: Vote::Yes,
         };
         app.execute_contract(Addr::unchecked(VOTER2), flex_addr.clone(), &yes_vote, &[])
             .unwrap();
-        assert_eq!(prop_status(&app), Status::Passed);
+        assert_eq!(prop_status(&app), Status::Open);
 
         // new proposal can be passed single-handedly by newbie
         let proposal = pay_somebody_proposal();
@@ -1498,13 +1618,13 @@ mod tests {
         let init_funds = coins(10, "BTC");
         let mut app = mock_app(&init_funds);
 
-        // 33% required for quora, which is 5 of the initial 15
-        // 50% yes required to pass early (8 of the initial 15)
+        // 33% required for quora, which is 8 of the initial 24
+        // 50% yes required to pass early (12 of the initial 24)
         let voting_period = Duration::Time(20000);
         let (flex_addr, group_addr) = setup_test_case(
             &mut app,
             Threshold::ThresholdQuorum {
-                threshold: Decimal::percent(50),
+                threshold: Decimal::percent(51),
                 quorum: Decimal::percent(33),
             },
             voting_period,
@@ -1528,17 +1648,17 @@ mod tests {
             prop.status
         };
 
-        // 3/5 votes - not expired
+        // 3/12 votes - not expired
         assert_eq!(prop_status(&app), Status::Open);
 
         // a few blocks later...
         app.update_block(|block| block.height += 2);
 
-        // admin changes the group (3 -> 0, 2 -> 7, 0 -> 15) - total = 32, require 11 to pass
+        // admin changes the group (3 -> 0, 2 -> 9, 0 -> 28) - total = 55, require 28 to pass
         let newbie: &str = "newbie";
         let update_msg = cw4_group::msg::ExecuteMsg::UpdateMembers {
             remove: vec![VOTER3.into()],
-            add: vec![member(VOTER2, 7), member(newbie, 15)],
+            add: vec![member(VOTER2, 9), member(newbie, 29)],
         };
         app.execute_contract(Addr::unchecked(OWNER), group_addr, &update_msg, &[])
             .unwrap();
@@ -1546,20 +1666,20 @@ mod tests {
         // a few blocks later...
         app.update_block(|block| block.height += 3);
 
-        // VOTER2 votes no, according to original weights: 3 yes, 2 no, 5 total (will pass when expired)
-        // with updated weights, it would be 3 yes, 7 no, 10 total (will fail when expired)
+        // VOTER2 votes yes, according to original weights: 3 yes, 2 no, 5 total (will fail when expired)
+        // with updated weights, it would be 3 yes, 9 yes, 11 total (will pass when expired)
         let yes_vote = ExecuteMsg::Vote {
             proposal_id,
-            vote: Vote::No,
+            vote: Vote::Yes,
         };
         app.execute_contract(Addr::unchecked(VOTER2), flex_addr.clone(), &yes_vote, &[])
             .unwrap();
         // not expired yet
         assert_eq!(prop_status(&app), Status::Open);
 
-        // wait until the vote is over, and see it was passed (met quorum, and threshold of voters)
+        // wait until the vote is over, and see it was rejected
         app.update_block(expire(voting_period));
-        assert_eq!(prop_status(&app), Status::Passed);
+        assert_eq!(prop_status(&app), Status::Rejected);
     }
 
     #[test]
